<?php

namespace Controllers;

use Services\ReportWorkflowService as Service;
use Controllers\ReportOrderController as ReportXML;
use Controllers\AFProcessController as AFProcess;
use Services\PushNotificationService;

class ReportWorkflowController
{
    public function __construct(Service $service, ReportXML $reportXML, AFProcess $afProcess, $pushNotificationService)
    {
        $this->service   = $service;
        $this->reportXML = $reportXML;
        $this->afProcess = $afProcess;
        $this->pushNotificationService = $pushNotificationService;
    }

    public function sendApprovalReport($orderData, $subjectParam, $justifyMessage, $cdOperFrom)
    {
        $orderCode = $orderData['NRAUTOFORN'];
        $supplierCode = $orderData['CDFORNECED'];
        self::sendApprovalPushNotification($orderData);
        if ($subjectParam == 'APROVACAO_ALCADA') {
            self::sendApprovalEmail($orderData, $subjectParam, $justifyMessage, $cdOperFrom);
        } else if ($subjectParam == 'LIBERACAO_DEMANDA') {
            $this->reportXML->reportOrderSuccess($orderCode, $supplierCode);
            /**
             * Envio de e-mail para todos os aprovadores ao liberar a demanda removido
             * devido a solicitação da Sapore.
             */
            // self::sendFinalApprovalEmail($orderData, $subjectParam, $justifyMessage, $cdOperFrom);
        }
    }

    private function sendApprovalEmail($orderData, $subject, $justify, $cdOperFrom)
    {
        $approver = $this->service->getApprover($orderData);
        if (!empty($approver)) {
            $orderApproverCode = $approver[0]['NRALCADAGERFILI'];
            $emails = $this->service->getUserAddress($orderApproverCode);
            self::sendEmail($orderData, $emails, $subject, $justify, $cdOperFrom);
        }
    }

    public function sendApprovalPushNotification($orderData)
    {
        $approver = $this->service->getApprover($orderData);
        if (!empty($approver)) {
            $approverID = $approver[0]['CDOPERADOR'];
            $devicesData = $this->service->getApproverDevices($approverID);
            $filial = $orderData['NMFILIAL'];
            $nrAutoForn = $orderData['NRAUTOFORN'];
            $data = array(
                "NRAPROVALCADA" => $orderData['NRAPROVALCADA'],
                "NRCONFIGALCADA" => $orderData['NRCONFIGALCADA'],
                "NMALCADA" => $orderData['NMALCADA'],
                "CDFILDEST" => $orderData['CDFILDEST'],
                "CDFILLANC" => $orderData['CDFILLANC'],
                "NMFILIAL" => $orderData['NMFILIAL'],
                "IDTPDEMANDA" => $orderData['IDTPDEMANDA'],
                "IDCLASSIFIC" => $orderData['IDCLASSIFIC'],
                "VRTOTALAF" => $orderData['VRTOTALAF'],
                "VRAPROVCMP" => $orderData['VRAPROVCMP'],
                "CDOPERADOR" => $orderData['CDOPERADOR'],
                "DTINCLUSAO" => $orderData['DTINCLUSAO'],
                "NRORG" => $orderData['NRORG'],
                "NRDEMANDA" => $orderData['NRAUTOFORN'],
                "APROVADOR" => $approverID,
                "IDAPROVADO" => "PENDENTE",
            );

            foreach ($devicesData as $deviceData) {
                try {
                    $this->pushNotificationService->sendPushNotification(
                        $deviceData["TOKEN"],
                        array(
                            'title' => "Nova AF pendente.",
                            'body' => "Número: {$nrAutoForn}, Unidade: {$filial}",
                            'data' => $data
                        )
                    );
                } catch (\Exception $e) {
                    // TODO: handle fcm exceptions
                }
            }
        }
    }


    private function sendFinalApprovalEmail($orderData, $subject, $justify, $cdOperFrom)
    {
        $emails = self::getEmailsFinalAprov($orderData['NRCONFIGALCADA'], $orderData['CDFILDEST']);
        self::sendEmail($orderData, $emails, $subject, $justify, $cdOperFrom);
    }

    private function getEmailsFinalAprov($NRCONFIGALCADA, $CDFILDEST)
    {
        $aprovadores = $this->service->getTodosAprovadores($NRCONFIGALCADA, $CDFILDEST);
        $aprovadoresRelacionados = $this->service->getTodosAprovadoresRelacionados($NRCONFIGALCADA, $CDFILDEST);
        if (!empty($aprovadoresRelacionados)) {
            return array_merge($aprovadores, $aprovadoresRelacionados);
        } else {
            return $aprovadores;
        }
    }

    public function sendRepproveReport($orderData, $subjectParam, $justifyMessage, $cdOperFrom)
    {
        $configOrderCode = $orderData['NRCONFIGALCADA'];
        $orderCode = $orderData['NRAUTOFORN'];
        $cdFildest = $orderData['CDFILDEST'];
        $emails = self::getEmailsToRepprove($configOrderCode, $cdFildest, $orderCode);
        self::sendEmail($orderData, $emails, $subjectParam, $justifyMessage, $cdOperFrom);
        self::sendErrorReport($orderData);
    }

    private function getEmailsToRepprove($configOrderCode, $cdFildest, $orderCode)
    {
        $launchUserCode = $this->service->getUserLaunchCode($orderCode, $cdFildest)['CDOPERLANCAF'];
        $launchUser     = $this->service->getUserLaunch($launchUserCode);
        $relatUser      = $this->service->getUserRelat($configOrderCode, $cdFildest);
        $gerUser        = $this->service->getUserGer($configOrderCode, $cdFildest);
        $emails         = array_merge($launchUser, $relatUser, $gerUser);
        return $emails;
    }

    private function sendErrorReport($orderData)
    {
        $nrAutoForn = $orderData['NRAUTOFORN'];
        $numOrder = $this->reportXML->checkNumOrder($nrAutoForn);
        if (!empty($numOrder)) {
            $userName = self::getRepproveUser($orderData);
            $erroReport = "Pedido recusado por $userName.";
            $cdFildest = $orderData['CDFILDEST'];
            $orderDate = $orderData['DTAUTOFORN'];
            $cdSupplier = $orderData['CDFORNECED'];
            $this->afProcess->importOrderError($numOrder, $cdFildest, $orderDate, $erroReport);
            $this->reportXML->reportOrderError($numOrder, $cdSupplier);
        }
    }

    private function getRepproveUser($orderData)
    {
        $userCode = $orderData['CDOPERADOR'];
        $operFrom = $this->service->getUserFrom($userCode)[0];
        return $operFrom['NMOPERADOR'];
    }

    private function sendEmail($orderData, $emails, $subject, $justify, $cdOperFrom)
    {
        foreach ($emails as $email) {
            if (!empty($email['DSEMAILOPER'])) {
                $operFrom = $this->service->getUserFrom($cdOperFrom)[0];
                $this->service->sendEmail($operFrom, $email, $orderData, $subject, $justify);
            }
        }
    }
}
