<?php
namespace Modules\Cadastros\Controllers;

use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Error;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Setforms2\Shared\ORM\DateTime;

class CalendarioComprasController extends \Zeedhi\Framework\Controller\Report
{
	protected $connection; 
	protected $qr2Strategy;
	protected $stNomeRel;
    protected $stExtensao;
    protected $reportMapping = array ();
    protected $birtStrategy;
	
	public function __construct($reportService, $entityManager, $qr2Strategy, $newCode, $environment, $connection, $dataSourceManager, $birtStrategy)
	{
        parent::__construct($reportService, $environment);
        $this->entityManager     = $entityManager;
        $this->qr2Strategy       = $qr2Strategy;
        $this->reportService     = $reportService;
        $this->newCode           = $newCode;
        $this->environment       = $environment;
        $this->connection        = $connection;
        $this->dataSourceManager = $dataSourceManager;
        $this->birtStrategy      = $birtStrategy;
    }
    
    public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) 
    {
        $row = $request->getRow();
        $this->birtStrategy->setReportFormat(strtolower($row['QR2TYPE']));
        $this->stNomeRel = $this->getReportName($row);
        $row['NRORG'] = $this->environment->getNrOrg();
        
        if($row['STRATEGY'] == 'BIRT'){
            if($row['CDFILIAL'] != 'T'){
                $nrlistaForn = $this->criarListaValores($row['CDFILIAL']);
                $row['CDFILIAL'] = "_NRLISTA=".$nrlistaForn;
            }
            if($row['CDPERIENTR'] != 'T'){
                $nrlistaForn = $this->criarListaValores($row['CDPERIENTR']);
                $row['CDPERIENTR'] = "_NRLISTA=".$nrlistaForn;
            }
        }
        $this->reportMapping[$this->stNomeRel] =
                array (
                'strategy' => $row['STRATEGY'],
                'parameterMapping' =>
                array (
                    "LIMSEG"     => "P_IN_LIM_SEG",
                    "DTINI"      => "P_DTINI",
                    "DTFIM"      => "P_DTFIM",
                    "CDFILIAL"   => "L_LST_FILIAL",
                    "CDPERIENTR" => "L_LST_PERI",
                    "NRORG"      => "P_NRORG"
                ),
            );
    }
    
    protected function criarListaValores($lista){
        $nrlista = $this->newCode->getCodeByTableName('LISTA', $this->environment->getNrOrg());
        $valores = explode('_',$lista);
        try{
            foreach($valores as $key){
                $this->insertLista($nrlista, $key);
            }
            return $nrlista;
        } catch(\Exception $e){
            throw $e;
        }
    }

    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }

	public function save(DTO\Request $request, DTO\Response $response)
	{
		try {
    		$row = $request->getRow();

            $query = 'UPDATE FILIAL SET CDFILCENTCMP = NULL WHERE CDFILIAL = :CDFILIAL';
            $stmt = $this->connection->prepare($query);
            $stmt->execute(array(
                "CDFILIAL" => $data[0]['CDFILIAL']
            ));
            $this->connection->commit();
		} catch (\Exception $e) {
		    $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
		}
	}

	public function isHoliday(DTO\Request $request, DTO\Response $response)
	{
	    $row = $request->getRow();
	    $query = "SELECT
                   DTFERICOMP
                FROM
                   FERICOMP
                WHERE
                   CDFILIAL   = :CDFILIAL AND
                   DTFERICOMP = :DTENTREGA";
        $stmt = $this->connection->prepare($query);
        try {
            $stmt->execute(array(
                "CDFILIAL"   => $row["CDFILIAL"],
                "DTENTREGA" => $row["DTENTREGA"]
            ));
            $holiday = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            $is_holiday = true;
            if(empty($holiday)){
                $is_holiday = false;
            }
            $response->addDataSet(new DataSet('IS_HOLIDAY', array($is_holiday)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
	}

	public function hadDateEnt(DTO\Request $request, DTO\Response $response)
	{
	    $row = $request->getRow();
	    $query = "SELECT 
                    CDPERIENTR
                  FROM   
                    DIASENTR
                  WHERE
                    (CDPERIENTR = :CDPERIENTR)";
        $stmt = $this->connection->prepare($query);
        $this->connection->beginTransaction();
        try {
            $stmt->execute(array(
                "CDPERIENTR" => $row["CDPERIENTR"]
            ));
            $this->connection->commit();
            $dateEnt = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            $had_date = true;
            if(empty($dateEnt)){
                $had_date = false;
            }
            $response->addDataSet(new DataSet('HAD_DTENT', array($had_date)));
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
            $this->connection->rollback();
        }
	}
	
    public function saveDate(DTO\Request $request, DTO\Response $response)
    {
        $row = $request->getRow();
        $queryCountDate = "SELECT COUNT(*) AS DATA
                            FROM
                               CALECOMP
                            WHERE
                               CDFILIAL   = :CDFILIAL   	 AND
                               CDPERIENTR = :CDPERIENTR      AND
                               DTENTREGA  > :DTENTREGA";
                               
        try {
            $stmt = $this->connection->prepare($queryCountDate);
        
            $stmt->execute(array(
                "CDFILIAL"   => $row["CDFILIAL"],
                "CDPERIENTR" => $row["CDPERIENTR"],
                "DTENTREGA"  => $row["DTENTREGA"]
            ));
            $count = $stmt->fetchAll(\PDO::FETCH_ASSOC)[0]["DATA"];
            if ($count == 0){
                $callback = $this->calcEntProxMes($row);
                if($callback && is_string($callback)){
                    $dtantproxent = date("d/m/Y", strtotime($callback));
                    $error = $this->updateCalecomp($row);
                    if($error){
                        throw new \Exception($error->getMessage());
                    }
                    $row["DTANTPROXENT"] = $dtantproxent;
                    $this->insertCalecomp($row);
                } else {
                    throw new \Exception($callback->getMessage());
                }
            } else {
                $callback = $this->getDateAntProxEnt($row);
                if($callback && is_string($callback)){
                    $row["DTANTPROXENT"] = explode(" ", $callback)[0];
                    $error = $this->updateCalecomp($row);
                    if($error){
                        throw new \Exception($error->getMessage());
                    }
                    $this->insertCalecomp($row);
                } else {
                    $response->setError(new Error($callback->getMessage(), 500));
                }
            }
        } catch (\Exception $e) {
            $response->setError(new Error($e->getMessage(), 500));
        }
	}
	
	private function getDateAntProxEnt($row)
	{
	    $querySelectDate = "SELECT
                               DTENTREGA-1 AS DT_ANT
                            FROM
                               CALECOMP
                            WHERE
                               CDFILIAL   = :CDFILIAL   			 AND
                               CDPERIENTR = :CDPERIENTR 			 AND
                               DTENTREGA  > :DTENTREGA
                            ORDER BY
                               1";
        try {
            $stmt = $this->connection->prepare($querySelectDate);
            $stmt->execute(array(
                "CDFILIAL"   => $row["CDFILIAL"],
                "CDPERIENTR" => $row["CDPERIENTR"],
                "DTENTREGA"  => $row["DTENTREGA"]
            ));
            return $stmt->fetchAll(\PDO::FETCH_ASSOC)[0]["DT_ANT"];
        } catch (\Exception $e) {
            return $e;
        }
        
	}
	
	private function insertCalecomp($row)
	{
	    $queryInsertCalecomp = "INSERT INTO
	                             CALECOMP
                                    (CDFILIAL, CDPERIENTR, DTENTREGA, DTANTPROXENT)
                                 VALUES
                                    (:CDFILIAL, :CDPERIENTR, :DTENTREGA, :DTANTPROXENT)";
        $this->connection->beginTransaction();
        try {
            $stmt = $this->connection->prepare($queryInsertCalecomp);
            $stmt->execute(array(
                "CDFILIAL"     => $row["CDFILIAL"],
                "CDPERIENTR"   => $row["CDPERIENTR"],
                "DTENTREGA"    => $row["DTENTREGA"],
                "DTANTPROXENT" => $row["DTANTPROXENT"]
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw new \Exception($e->getMessage());
        }
	}
	
	private function updateCalecomp($row)
	{
	    $queryUpdate = "UPDATE
                           CALECOMP
                        SET
                           DTANTPROXENT = (TO_DATE(:DTENTREGA) - 1)
                        WHERE
                           0 <> (SELECT COUNT(DTENTREGA)
                                 FROM
                                    CALECOMP
                                 WHERE
                                    CDFILIAL   = :CDFILIAL   AND
                                    CDPERIENTR = :CDPERIENTR AND
                                    DTENTREGA  < TO_DATE(:DTENTREGA)) AND
                           CDFILIAL   = :CDFILIAL   AND
                           CDPERIENTR = :CDPERIENTR AND
                           DTENTREGA  = (SELECT MAX(DTENTREGA)
                                         FROM
                                            CALECOMP
                              			  WHERE
                                            CDFILIAL   = :CDFILIAL   AND
                        				  CDPERIENTR = :CDPERIENTR AND
                        				  DTENTREGA  < TO_DATE(:DTENTREGA))";
        try {
            $this->connection->beginTransaction();
            $stmt = $this->connection->prepare($queryUpdate);
            $stmt->execute(array(
                "CDFILIAL"   => $row["CDFILIAL"],
                "CDPERIENTR" => $row["CDPERIENTR"],
                "DTENTREGA"  => $row["DTENTREGA"]
            ));
            $this->connection->commit();
            return false;
        } catch (\Exception $e) {
            $this->connection->rollback();
            return $e;
        }
	}
	
	private function calcEntProxMes($row)
	{
        try{
            $procedure = new StoredProcedure($this->connection, 'CALC_PROX_ENTR_MES');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT,  $row['CDFILIAL'],   Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPERIENTR',  Param::PARAM_INPUT,  $row['CDPERIENTR'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL',   Param::PARAM_INPUT,  $row["DTENTREGA"],  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINIAUX',    Param::PARAM_OUTPUT, NULL,               Param::PARAM_TYPE_STR,  10));
            $callback = $procedure->execute();
            return $callback["P_DTINIAUX"];
        }catch(\Exception $e){
            return $e;
        }
    }
	
	public function calcDtEntrega(DTO\Request $request, DTO\Response $response)
	{
	    $row = $request->getRow();
	    $dtProcessa = $this->getDtProc($row);
	    try {
            $procedure = new StoredProcedure($this->connection, 'CALC_DATA_ENTR_MES');
            $procedure->addParam(new Param('P_CDFILIAL',    Param::PARAM_INPUT, $row['CDFILIAL'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDPERIENTR',    Param::PARAM_INPUT, $row['CDPERIENTR'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL',    Param::PARAM_INPUT, $row['DTENTREGA'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRMESES',    Param::PARAM_INPUT, $row['NRMESES'], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTPROCESSA',    Param::PARAM_INPUT, $dtProcessa, Param::PARAM_TYPE_STR));
            $procedure->execute();
	    } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
	    }
    }
    
	public function deleteDay(DTO\Request $request, DTO\Response $response)
	{
	    $row = $request->getRow();
        $queryUpdateCalecomp =   "UPDATE
                                    CALECOMP
                                  SET
                                   DTANTPROXENT = (SELECT DTANTPROXENT
                                                     FROM   CALECOMP
                                                    WHERE  CDFILIAL   = :CDFILIAL   AND
                                					       CDPERIENTR = :CDPERIENTR AND
                                					       DTENTREGA  = :DTENTREGA)
                                   WHERE
                                     0 <> (SELECT COUNT(DTENTREGA)
                                             FROM   CALECOMP
                                             WHERE  CDFILIAL   = :CDFILIAL    AND
                                                    CDPERIENTR = :CDPERIENTR  AND
                                                    DTENTREGA  < :DTENTREGA
                                           )  AND
                                     CDFILIAL   = :CDFILIAL   AND
                                     CDPERIENTR = :CDPERIENTR AND
                                     DTENTREGA  = (SELECT
                                                     MAX(DTENTREGA)
                                                   FROM
                                                     CALECOMP
                                         		   WHERE
                                                     CDFILIAL = :CDFILIAL   	 AND
                            						 CDPERIENTR = :CDPERIENTR 	 AND
                                    				 DTENTREGA  < :DTENTREGA  	
                                    				)";
        $queryDeleteCalecomp = "DELETE
                           CALECOMP
                        WHERE
                           CDFILIAL   = :CDFILIAL    AND
                           CDPERIENTR = :CDPERIENTR  AND
                           DTENTREGA  = :DTENTREGA";
        
       $this->connection->beginTransaction();
        try{
            $stmt = $this->connection->prepare($queryUpdateCalecomp);
            $stmt->execute(array(
                "CDPERIENTR" => $row["CDPERIENTR"],
                "CDFILIAL"   => $row["CDFILIAL"],
                "DTENTREGA"  => $row["DTENTREGA"]
            ));
            $this->connection->commit();
            $this->connection->beginTransaction();
            $stmt = $this->connection->prepare($queryDeleteCalecomp);
            $stmt->execute(array(
                "CDPERIENTR" => $row["CDPERIENTR"],
                "CDFILIAL"   => $row["CDFILIAL"],
                "DTENTREGA"  => $row["DTENTREGA"]
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
	}
	
	public function deleteMonth(DTO\Request $request, DTO\Response $response)
	{
	    $row = $request->getRow();
	    $queryUpdate = "UPDATE
                           CALECOMP
                        SET
                           DTANTPROXENT = (SELECT
                                              DTANTPROXENT
                                           FROM
                                              CALECOMP
                                           WHERE
                                              CDFILIAL   = :CDFILIAL   AND
                                              CDPERIENTR = :CDPERIENTR AND
                                              DTENTREGA  = (SELECT
                                                               MAX(DTENTREGA)
                                                            FROM
                                                               CALECOMP
                                                            WHERE
                                                               CDFILIAL   = :CDFILIAL   AND
                                                               CDPERIENTR = :CDPERIENTR
                                                            GROUP BY
                                                               TO_CHAR(DTENTREGA,'MM/YYYY')
                                                            HAVING
                                                               TO_CHAR(MAX(DTENTREGA),'MM/YYYY') = :DTENTREGAINI))
                        WHERE
                           0 <> (SELECT COUNT(DTENTREGA)
                                 FROM
                                    CALECOMP
                                 WHERE
                                    CDFILIAL   = :CDFILIAL   AND
                                    CDPERIENTR = :CDPERIENTR AND
                                    DTENTREGA  < (SELECT
                                                     MAX(DTENTREGA)
                                                  FROM
                                                     CALECOMP
                                                  WHERE
                                                     CDFILIAL   = :CDFILIAL   AND
                                                     CDPERIENTR = :CDPERIENTR
                                                  GROUP BY
                                                     TO_CHAR(DTENTREGA,'MM/YYYY')
                                                  HAVING
                                                     TO_CHAR(MAX(DTENTREGA),'MM/YYYY') = :DTENTREGAINI)) AND
                           CDFILIAL   = :CDFILIAL   AND
                           CDPERIENTR = :CDPERIENTR AND
                           DTENTREGA  = (SELECT
                                            MAX(DTENTREGA)
                                         FROM
                                            CALECOMP
                                   		  WHERE
                                            CDFILIAL   = :CDFILIAL   AND
                        		              CDPERIENTR = :CDPERIENTR AND
                        		              DTENTREGA  < (SELECT
                                                             MIN(DTENTREGA)
                                                          FROM
                                                             CALECOMP
                                                          WHERE
                                                             CDFILIAL   = :CDFILIAL   AND
                                                             CDPERIENTR = :CDPERIENTR AND
                                                             TO_DATE(DTENTREGA) >= :DATAPROC
                                                          GROUP BY
                                                             TO_CHAR(DTENTREGA,'MM/YYYY')
                                                          HAVING
                                                             TO_CHAR(MAX(DTENTREGA),'MM/YYYY') = :DTENTREGAINI))";
        $queryDelete = "DELETE
                           CALECOMP
                        WHERE
                           CDFILIAL   =  :CDFILIAL      AND
                           CDPERIENTR =  :CDPERIENTR    AND
                           TO_DATE(DTENTREGA) >= :DATAPROC AND
                           TO_CHAR(DTENTREGA,'MM/YYYY') = :DTENTREGA";
        try {
            $this->connection->beginTransaction();
            $stmt = $this->connection->prepare($queryUpdate);
            $stmt->execute(array(
                "CDPERIENTR"   => $row["CDPERIENTR"],
                "CDFILIAL"     => $row["CDFILIAL"],
                "DTENTREGAINI" => $row["DTENTREGAINI"],
                "DATAPROC"     => $row["DTPROCESSA"]
            ));
            
            $this->connection->commit();
            $this->connection->beginTransaction();
            $stmt = $this->connection->prepare($queryDelete);
            $stmt->execute(array(
                "CDPERIENTR" => $row["CDPERIENTR"],
                "CDFILIAL"   => $row["CDFILIAL"], 
                "DATAPROC"   => $row["DTPROCESSA"], 
                "DTENTREGA"  => $row["DTENTREGAINI"]
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
	}
	
	private function getDtProc($row)
	{
	    $queryDtProcessa = "SELECT DTPROCESSA
                                FROM
                                  (
                                    SELECT (
                                      CASE PARAMGERAL.IDUTILDTPROC
                                        WHEN 'N'
                                        THEN SYSDATE
                                        ELSE NVL(PARAMFILIAL.DTPROCESSA, SYSDATE)
                                      END) DTPROCESSA
                                    FROM PARAMGERAL,
                                      PARAMFILIAL
                                    WHERE PARAMFILIAL.CDFILIAL = :CDFILIAL
                                  )";
        $stmt = $this->connection->prepare($queryDtProcessa);
        $stmt->execute(array(
            "CDFILIAL"   => $row["CDFILIAL"]
        ));
        return $stmt->fetchAll(\PDO::FETCH_ASSOC)[0]["DTPROCESSA"];
	}
	
	public function deleteYear(DTO\Request $request, DTO\Response $response)
	{
	    $row = $request->getRow();
	    $queryUpdateCalecomp = "UPDATE
                                   CALECOMP
                                SET
                                   DTANTPROXENT = (SELECT
                                                      DTANTPROXENT
                                                   FROM
                                                      CALECOMP
                                                   WHERE
                                                      CDFILIAL   = :CDFILIAL   AND
                                            		    CDPERIENTR = :CDPERIENTR AND
                                		                DTENTREGA  = (SELECT
                                                                       MAX(DTENTREGA)
                                                                    FROM
                                                                       CALECOMP
                                                                    WHERE
                                                                       CDFILIAL   = :CDFILIAL   AND
                                                                       CDPERIENTR = :CDPERIENTR
                                                                    GROUP BY
                                                                       TO_CHAR(DTENTREGA,'YYYY')
                                                                    HAVING
                                                                       TO_CHAR(MAX(DTENTREGA),'YYYY') = :DTENTREGAINI))
                                WHERE
                                   0 <> (SELECT COUNT(DTENTREGA)
                                         FROM
                                            CALECOMP
                                         WHERE
                                            CDFILIAL   = :CDFILIAL   AND
                                            CDPERIENTR = :CDPERIENTR AND
                                            DTENTREGA  < (SELECT
                                                             MAX(DTENTREGA)
                                                          FROM
                                                             CALECOMP
                                                          WHERE
                                                             CDFILIAL   = :CDFILIAL   AND
                                                             CDPERIENTR = :CDPERIENTR
                                                          GROUP BY
                                                             TO_CHAR(DTENTREGA,'YYYY')
                                                          HAVING
                                                             TO_CHAR(MAX(DTENTREGA),'YYYY') = :DTENTREGAINI)) AND
                                   CDFILIAL   = :CDFILIAL   AND
                                   CDPERIENTR = :CDPERIENTR AND
                                   DTENTREGA  = (SELECT
                                                    MAX(DTENTREGA)
                                                 FROM
                                                    CALECOMP
                                     		        WHERE
                                                    CDFILIAL   = :CDFILIAL   AND
                                		              CDPERIENTR = :CDPERIENTR AND
                                		              DTENTREGA  < (SELECT
                                                                     MIN(DTENTREGA)
                                                                  FROM
                                                                     CALECOMP
                                                                  WHERE
                                                                     CDFILIAL   = :CDFILIAL   AND
                                                                     CDPERIENTR = :CDPERIENTR AND
                                                                     TO_DATE(DTENTREGA) >= :DATAPROC
                                                                  GROUP BY
                                                                     TO_CHAR(DTENTREGA,'YYYY')
                                                                  HAVING
                                                                     TO_CHAR(MAX(DTENTREGA),'YYYY') = :DTENTREGAINI))";
        $deleteByYearCalecomp = "DELETE
                                   CALECOMP
                                WHERE
                                   CDFILIAL   =  :CDFILIAL      AND
                                   CDPERIENTR =  :CDPERIENTR    AND
                                   TO_DATE(DTENTREGA) >= :DATAPROC AND
                                   TO_CHAR(DTENTREGA,'YYYY') = :DTENTREGA";
        try {
            $this->connection->beginTransaction();
            $stmt = $this->connection->prepare($queryUpdateCalecomp);
            $stmt->execute(array(
                "CDFILIAL"     => $row["CDFILIAL"],
                "DTENTREGAINI" => $row["DTENTREGA"],
                "DATAPROC"     => $row["DTPROCESSA"],
                "CDPERIENTR"   => $row["CDPERIENTR"] 
            ));
            $this->connection->commit();
            $this->connection->beginTransaction();
            $stmt = $this->connection->prepare($deleteByYearCalecomp);
            $stmt->execute(array(
                "CDFILIAL"   => $row["CDFILIAL"],
                "CDPERIENTR" => $row["CDPERIENTR"],
                "DATAPROC"   => $row["DTPROCESSA"],
                "DTENTREGA"  => $row["DTENTREGA"] 
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
        }
	} 
	
	public function exportCalendar(DTO\Request $request, DTO\Response $response)
	{
	    try {
	        $row = $request->getRow();
	        $hadCalendarEntr = $this->hadCalendarEntr($row);
	        if(!$hadCalendarEntr){
	            $response->setError(new Error("CALECOMP_NO_DATA_FILTERED", 500));
	        } else {
    	        $this->deleteCalecompForExport($row);
    	        $this->insertCalecompForExport($row);
    	        $rowForLoop = array(
    	            'CDFILDEST' => '',
    	            'CDPERIENTR' => '',
    	            'DTENTRINI' => $row['DTENTRINI'],
    	            'DTENTREGA' => $row['DTENTRINI']
    	        );
    	        $filiDest = explode('_', $row['CDFILDEST']);
    	        $periodicidade = explode('_', $row['CDPERIENTR']);
    	        foreach($filiDest as $filial){
    	            $rowForLoop['CDFILDEST'] = $filial;
    	            foreach($periodicidade as $period){
    	                $rowForLoop['CDPERIENTR'] = $period;
            	        $maxDtEntrega = $this->getMaxDtEntrega($rowForLoop);
            	        if($maxDtEntrega){
            	            $minDtEntrega = $this->getMinDtEntrega($rowForLoop);
            	            $this->updateCalecomp($rowForLoop);
            	        }
    	            }
    	        }
	        }
	    } catch (\Exception $e) {
	        $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
	    }
	}
	
	private function deleteCalecompForExport($row)
	{
	    $queryDeleteCalecomp = "DELETE FROM CALECOMP
                                WHERE (INSTR(:CDFILDEST, CDFILIAL) <> 0 OR :TCDFILDEST = 'T')
                                  AND (CDFILIAL <> :CDFILORIG)
                                  AND (INSTR(:CDPERIENTR, CDPERIENTR) > 0 OR :CDPERIENTR = 'T')
                                  AND (DTENTREGA BETWEEN :DTENTRINI AND :DTENTRFIN)
                                  AND (EXISTS (SELECT 1
                                               FROM FILIOPER
                                               WHERE FILIOPER.CDOPERADOR = :CDOPERADOR
                                                 AND FILIOPER.CDFILIAL   = CALECOMP.CDFILIAL))
                                  AND ((:CDGRUPFILI IS NULL)
                                       OR
                                       (EXISTS (SELECT 1
                                                FROM FILGRUPFILI
                                                WHERE FILGRUPFILI.CDGRUPFILI = :CDGRUPFILI
                                                  AND FILGRUPFILI.CDFILIAL   = CALECOMP.CDFILIAL))
                                      )";
        try {
            $this->connection->beginTransaction();
            $stmt = $this->connection->prepare($queryDeleteCalecomp);
            $stmt->execute(array(
                'CDFILDEST'  => $row['CDFILDEST'],
                'TCDFILDEST' => $row['TCDFILDEST'],
                'CDFILORIG'  => $row['CDFILORIG'],
                'CDPERIENTR' => $row['CDPERIENTR'],
                'DTENTRINI'  => $row['DTENTRINI'],
                'DTENTRFIN'  => $row['DTENTRFIN'],
                'CDOPERADOR' => $row['CDOPERADOR'],
                'CDGRUPFILI' => $row['CDGRUPFILI']
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw new \Exception($e->getMessage());
        }
	}

	private function insertCalecompForExport($row)
	{
	    $queryInsertCalecomp = "INSERT INTO CALECOMP
                                   (CDFILIAL, CDPERIENTR, DTENTREGA, DTANTPROXENT)
                                (SELECT
                                   A.CDFILIAL, B.CDPERIENTR, B.DTENTREGA, B.DTANTPROXENT
                                FROM
                                   (SELECT
                                       CDFILIAL
                                    FROM
                                       FILIAL
                                    WHERE
                                       (INSTR(:CDFILDEST,CDFILIAL) <> 0 OR :TCDFILDEST = 'T') AND
                                       (CDFILIAL <> :CDFILORIG)) A,
                                       CALECOMP B
                                WHERE (B.CDFILIAL = :CDFILORIG)
                                  AND (INSTR(:CDPERIENTR, B.CDPERIENTR) > 0 OR :CDPERIENTR = 'T')
                                  AND (DTENTREGA BETWEEN :DTENTRINI AND :DTENTRFIN)
                                  AND (EXISTS (SELECT 1
                                               FROM FILIOPER
                                               WHERE FILIOPER.CDOPERADOR = :CDOPERADOR
                                                 AND FILIOPER.CDFILIAL   = A.CDFILIAL))
                                  AND ((:CDGRUPFILI IS NULL)
                                       OR
                                       (EXISTS (SELECT 1
                                                FROM FILGRUPFILI
                                                WHERE FILGRUPFILI.CDGRUPFILI = :CDGRUPFILI
                                                  AND FILGRUPFILI.CDFILIAL   = A.CDFILIAL))
                                      )
                                )";
        try {
            $this->connection->beginTransaction();
            $stmt = $this->connection->prepare($queryInsertCalecomp);
            $stmt->execute(array(
                'CDFILDEST'  => $row['CDFILDEST'],
                'TCDFILDEST' => $row['TCDFILDEST'],
                'CDFILORIG'  => $row['CDFILORIG'],
                'CDPERIENTR' => $row['CDPERIENTR'],
                'DTENTRINI'  => $row['DTENTRINI'],
                'DTENTRFIN'  => $row['DTENTRFIN'],
                'CDOPERADOR' => $row['CDOPERADOR'],
                'CDGRUPFILI' => $row['CDGRUPFILI']
            ));
            $this->connection->commit();
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw new \Exception($e->getMessage());
        }
	}
	
	private function getMaxDtEntrega($row)
	{
	    $queryMaxDt = "SELECT MAX(DTENTREGA) ENTREGA
                          FROM CALECOMP
                       WHERE CDFILIAL   = :CDFILDEST
                          AND CDPERIENTR = :CDPERIENTR
                          AND DTENTREGA  < :DTENTRINI";
        try {
            $stmt = $this->connection->prepare($queryMaxDt);
            $stmt->execute(array(
                'CDFILDEST'  => $row['CDFILDEST'],
                'CDPERIENTR' => $row['CDPERIENTR'],
                'DTENTRINI'  => $row['DTENTRINI'], 
            ));
            return $stmt->fetchAll(\PDO::FETCH_ASSOC)[0]['ENTREGA'];
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw new \Exception($e->getMessage());
        }
	}
	
	private function getMinDtEntrega($row)
	{
	    $queryMinDt = "SELECT MIN(DTENTREGA) AS ENTREGA
                          FROM CALECOMP
                       WHERE  CDFILIAL    = :CDFILDEST
                          AND  CDPERIENTR  = :CDPERIENTR
                          AND  DTENTREGA  >= :DTENTRINI";
        try {
            $this->connection->beginTransaction();
            $stmt = $this->connection->prepare($queryMinDt);
            $stmt->execute(array(
                'CDFILDEST'  => $row['CDFILDEST'],
                'CDPERIENTR' => $row['CDPERIENTR'], 
                'DTENTRINI'  => $row['DTENTRINI']
            ));
            $this->connection->commit();
            return $stmt->fetchAll(\PDO::FETCH_ASSOC)[0]['ENTREGA'];
        } catch (\Exception $e) {
            $this->connection->rollback();
            throw new \Exception($e->getMessage());
        }
	}
	
	private function hadCalendarEntr($row)
	{
	    $queryHadCalendarEntr = "SELECT
                                   CDFILIAL
                                FROM
                                   CALECOMP
                                WHERE (CDFILIAL = :CDFILORIG)
                                  AND (INSTR(:CDPERIENTR, CDPERIENTR) > 0 OR :CDPERIENTR = 'T')
                                  AND (DTENTREGA BETWEEN :DTENTRINI AND :DTENTRFIN)";
        try {
            $stmt = $this->connection->prepare($queryHadCalendarEntr);
            $stmt->execute(array(
                'CDFILORIG'  => $row['CDFILORIG'], 
                'CDPERIENTR' => $row['CDPERIENTR'], 
                'DTENTRINI'  => $row['DTENTRINI'],
                'DTENTRFIN'  => $row['DTENTRFIN']
            ));
            $hadCalendarEntr = $stmt->fetchAll(\PDO::FETCH_ASSOC);
            if(!empty($hadCalendarEntr)){
                return true;
            } else {
                return false;
            }
        } catch (\Exception $e) {
            throw new \Exception($e->getMessage());
        }
	}
}