<?php

namespace Services;

use Util\SupUtilities;
use \Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use \Zeedhi\Framework\DB\StoredProcedure\Param;

class AprovacaoAlcadaService
{
    public function __construct(SupUtilities $supUtilities, $environment, $connection)
    {
        $this->supUtilities = $supUtilities;
        $this->environment  = $environment;
        $this->connection = $connection;
    }

    public function updateWorkflow($approveCode, $orderTag, $justifyMessage, $userLog)
    {
        $query = "UPDATE APROVALCADA SET IDAPROVADO = :P_IDAPROVADO, DSJUSTIFICA = :P_DSJUSTIFICA, DTAPROVACAO = SYSDATE, CDOPERAPROV = :P_CDOPERAPROV WHERE NRAPROVALCADA = :P_NRAPROVALCADA";
        $params = [
            "P_NRAPROVALCADA" => $approveCode,
            "P_IDAPROVADO" => $orderTag,
            "P_DSJUSTIFICA" => $justifyMessage,
            "P_CDOPERAPROV" => $userLog,
        ];
        $this->supUtilities->crud($query, $params);
    }

    public function getIdSisDos()
    {
        $sql = "SELECT IDSISDOS 
                  FROM PARAMGERAL";

        return $this->supUtilities->crud($sql, []);
    }

    public function getApproverOrder($data)
    {
        $sql = "SELECT 
                    IDEXIGETAPROVE
                FROM 
                    CONFIGALCADA 
                WHERE 
                    NRCONFIGALCADA = :NRCONFIGALCADA";

        $params = [
            "NRCONFIGALCADA" => $data['NRCONFIGALCADA']
        ];

        return $this->supUtilities->crud($sql, $params);
    }

    public function getApprover($orderData)
    {
        $query = "SELECT NRALCADAGERFILI, CDOPERADOR, VRAPROVCMP, (SELECT MIN(B.VRAPROVCMP) FROM ALCADAGERFILI B WHERE B.CDFILIAL = ALCADAGERFILI.CDFILIAL AND B.NRCONFIGALCADA = ALCADAGERFILI.NRCONFIGALCADA AND B.VRAPROVCMP > ALCADAGERFILI.VRAPROVCMP) VRAPROVCMPMAX FROM ALCADAGERFILI WHERE CDFILIAL = :P_CDFILIAL AND NRCONFIGALCADA = :P_NRCONFIGALCADA AND VRAPROVCMP = (SELECT MIN(C.VRAPROVCMP) FROM ALCADAGERFILI C WHERE C.CDFILIAL = ALCADAGERFILI.CDFILIAL AND C.NRCONFIGALCADA = ALCADAGERFILI.NRCONFIGALCADA AND ((C.VRAPROVCMP > :P_VRAPROVCMP) OR :P_VRAPROVCMP = 'T')) ORDER BY VRAPROVCMP";
        $params = [
            "P_CDFILIAL" => $orderData['CDFILDEST'],
            "P_VRAPROVCMP" => $orderData['VRAPROVCMP'],
            "P_NRCONFIGALCADA" => $orderData['NRCONFIGALCADA'],
        ];
        return $this->supUtilities->crud($query, $params);
    }

    public function setApprover($userCode, $nrApprove)
    {
        $approvalCode = floatval($this->supUtilities->generateCode('APROVALCADA'));
        $query = "INSERT INTO APROVALCADA ( NRAPROVALCADA, NRCONFIGALCADA, CDFILDEST, CDFILLANC, NRDEMANDA, CDOPERADOR, IDTPDEMANDA, IDCLASSIFIC, IDOPERACAOALC, IDAPROVADO, DTINCLUSAO ) SELECT :P_NEWNRAPROVALCADA, NRCONFIGALCADA, CDFILDEST, CDFILLANC, NRDEMANDA, :P_CDOPERADOR, IDTPDEMANDA, IDCLASSIFIC, IDOPERACAOALC, 'PENDENTE', SYSDATE FROM APROVALCADA WHERE NRAPROVALCADA =:P_NRAPROVALCADA";
        $params = [
            "P_NEWNRAPROVALCADA" => $approvalCode,
            "P_NRAPROVALCADA" => $nrApprove,
            "P_CDOPERADOR" => $userCode,
        ];
        return $this->supUtilities->crud($query, $params);
    }

    public function updateOrder($orderData, $orderTag)
    {
        $query = "UPDATE AUTOFORNE SET IDSITAFALCADA = :P_SITUACAO, NRCONFIGALCADA = :P_NRCONFIGALCADA, IDBAIXAAF = DECODE(:P_SITUACAO, 'APROVADA', null, DECODE(:P_SITUACAO, 'PENDENTE', 6, 5)) WHERE CDFILDEST = :P_CDFILDEST AND CDFILLANC = :P_CDFILLANC AND NRAUTOFORN = :P_NRAUTOFORN";
        $params = [
            "P_CDFILDEST" => $orderData['CDFILDEST'],
            "P_CDFILLANC" => $orderData['CDFILLANC'],
            "P_NRCONFIGALCADA" => $orderData['NRCONFIGALCADA'],
            "P_NRAUTOFORN" => $orderData['NRAUTOFORN'],
            "P_SITUACAO" => $orderTag,
        ];
        return $this->supUtilities->crud($query, $params);
    }

    public function getOrder($row)
    {
        $query = "SELECT C.NMALCADA, A.NRCONFIGALCADA, C.IDTIPOVRAPROV, A.NRAPROVALCADA, A.IDOPERACAOALC, A.IDCLASSIFIC, A.IDTPDEMANDA, AL.VRAPROVCMP, AL.NRALCADAGERFILI, F.NMFILIAL, AUT.* FROM AUTOFORNE AUT, APROVALCADA A, ALCADAGERFILI AL, CONFIGALCADA C, FILIAL F WHERE AUT.CDFILDEST =:P_CDFILDEST AND AUT.CDFILLANC =:P_CDFILLANC AND AUT.NRAUTOFORN =:P_NRAUTOFORN AND AUT.NRAUTOFORN = A.NRDEMANDA AND A.NRCONFIGALCADA = AL.NRCONFIGALCADA AND A.NRCONFIGALCADA = C.NRCONFIGALCADA AND AL.NRCONFIGALCADA = C.NRCONFIGALCADA AND A.CDFILDEST = AL.CDFILIAL AND A.CDFILDEST = F.CDFILIAL AND A.CDOPERADOR = AL.CDOPERADOR AND A.CDOPERADOR = :P_APROVADOR";
        $params = [
            "P_CDFILDEST" => $row['CDFILDEST'],
            "P_CDFILLANC" => $row['CDFILLANC'],
            "P_NRAUTOFORN" => $row['NRAUTOFORNE'],
            "P_APROVADOR" => $row['APROVADOR'],
        ];
        return $this->supUtilities->crud($query, $params)[0];
    }

    public function getApproversMaxValue($orderData)
    {
        $query = "SELECT MAX(VRAPROVCMP) VALORMAXIMO FROM ALCADAGERFILI WHERE CDFILIAL = :P_CDFILIAL AND NRCONFIGALCADA = :P_NRCONFIGALCADA";
        $params = [
            "P_CDFILIAL" => $orderData['CDFILDEST'],
            "P_NRCONFIGALCADA" => $orderData['NRCONFIGALCADA'],
        ];
        return $this->supUtilities->crud($query, $params)[0];
    }

    public function totalDiaAF($params)
    {
        try {
            $query = "SELECT SUM(VRTOTALAF) AS TOTAL FROM AUTOFORNE A, ITEMAUTF I WHERE A.CDFILDEST = :P_CDFILDEST AND A.NRAUTOFORN <> :P_NRAUTOFORN AND A.CDFILDEST = I.CDFILDEST AND A.NRAUTOFORN = I.NRAUTOFORN AND A.CDFILDEST||A.CDFORNECED||I.CDPRODUTO||I.DTPROGENTAF IN (SELECT ITEMAUTF.CDFILDEST||AUTOFORNE.CDFORNECED||ITEMAUTF.CDPRODUTO||ITEMAUTF.DTPROGENTAF FROM AUTOFORNE, ITEMAUTF WHERE AUTOFORNE.CDFILDEST = :P_CDFILDEST AND AUTOFORNE.CDFILLANC = :P_CDFILLANC AND AUTOFORNE.NRAUTOFORN = :P_NRAUTOFORN AND AUTOFORNE.CDFILDEST = ITEMAUTF.CDFILDEST AND AUTOFORNE.CDFILLANC = ITEMAUTF.CDFILLANC AND AUTOFORNE.NRAUTOFORN = ITEMAUTF.NRAUTOFORN)";
            $params = [
                "P_CDFILDEST"     => $params['CDFILDEST'],
                "P_CDFILLANC"     => $params['CDFILLANC'],
                "P_NRAUTOFORN"    => $params['NRDEMANDA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findProdutoOrcamento($params)
    {
        try {
            $query = "SELECT MAX(PROD.CDARVPROD) CDARVPROD, MAX(IDP.VRORCAMENTO) VRORCAMENTO
                        FROM   ORCAMENTOGOP O
                               INNER JOIN ITEMORCAGOP I
                                       ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                          AND I.IDTIPOORCAOP IN ( 'CONSUMO', 'DESPENTRADA',
                                                                  'DESPDIVERSA' )
                               INNER JOIN APROVAORCAGOP A
                                       ON A.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                               INNER JOIN ITORCAMESGOP IM
                                       ON IM.NRITEMORCAOP = I.NRITEMORCAOP
                               INNER JOIN ITDETALORCAMESGOP IDP
                                       ON IDP.NRITORCAMESGOP = IM.NRITORCAMESGOP
                               INNER JOIN PRODUTO PROD
                                       ON PROD.CDPRODUTO = IDP.CDPRODUTO
                        WHERE  O.CDFILIAL = :P_CDFILDEST
                               AND O.IDPERIODOORCA = :P_IDPERIODOORCA
                               AND O.NRPERIODOORCA = :P_NRPERIODOORCA
                               AND O.NRANOORCAGOP = TO_CHAR(TO_DATE(:P_DTPROGENTAF), 'YYYY')
                               AND TO_CHAR(IM.DTORCAMENTO, 'MM') = TO_CHAR(TO_DATE(:P_DTPROGENTAF), 'MM')
                               AND IDP.VRORCAMENTO IS NOT NULL
                               AND EXISTS(SELECT 1
                                          FROM   ITEMAUTF,
                                                 PRODUTO
                                          WHERE  ITEMAUTF.CDFILDEST = :P_CDFILDEST
                                                 AND ITEMAUTF.CDFILLANC = :P_CDFILLANC
                                                 AND ITEMAUTF.NRAUTOFORN = :P_NRAUTOFORN
                                                 AND ITEMAUTF.CDPRODUTO = PRODUTO.CDPRODUTO
                                                 AND PRODUTO.CDARVPROD LIKE PROD.CDARVPROD
                                                                            ||'%')";
            $params = [
                "P_IDPERIODOORCA"    => $params['IDPERIODOORCA'],
                "P_NRPERIODOORCA"    => $params['NRPERIODOORCA'],
                "P_DTPROGENTAF"      => $params['DTPROGENTAF'],
                "P_CDFILDEST"        => $params['CDFILDEST'],
                "P_CDFILLANC"        => $params['CDFILLANC'],
                "P_NRAUTOFORN"       => $params['NRDEMANDA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getTotalPeriodoAF($params)
    {
        try {
            $query = "SELECT
                            NVL(SUM(ITEMAUTF.VRPREUNIAF * ITEMAUTF.QTPROGENTAF), 0) VR_TOTAL_AF
                        FROM
                            ITEMAUTF,
                            AUTOFORNE,
                            PRODUTO
                        WHERE
                            AUTOFORNE.CDFILDEST                        = ITEMAUTF.CDFILDEST
                        AND AUTOFORNE.CDFILLANC                      = ITEMAUTF.CDFILLANC
                        AND AUTOFORNE.NRAUTOFORN                     = ITEMAUTF.NRAUTOFORN
                        AND TO_CHAR(ITEMAUTF.DTPROGENTAF, 'MM/YYYY') = TO_CHAR(TO_DATE(:P_DTPROGENTAF),
                            'MM/YYYY')
                        AND NVL(ITEMAUTF.IDBAIXITMAF, '1')    IN ( '1', '2' )
                        AND NVL(AUTOFORNE.IDBAIXAAF, '1') NOT IN ( '3', '4', '5' )
                        AND ITEMAUTF.CDFILDEST                 = :P_CDFILDEST
                        AND ITEMAUTF.NRAUTOFORN               <> :P_NRAUTOFORN
                        AND ITEMAUTF.CDPRODUTO                 = PRODUTO.CDPRODUTO
                        AND PRODUTO.CDARVPROD LIKE :P_CDARVPROD
                            ||'%' ";
            $params = [
                "P_DTPROGENTAF" => $params['DTPROGENTAF'],
                "P_CDFILDEST"   => $params['CDFILDEST'],
                "P_NRAUTOFORN"  => $params['NRDEMANDA'],
                "P_CDARVPROD"   => $params['CDARVPROD'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function getValorDemanda($params)
    {
        try {
            $query = "SELECT VALOR_DEMANDA(:P_CDFILDEST, :P_NRAUTOFORN, :P_IDPERIODOORCA, :P_NRPERIODOORCA, :P_DTPROGENTAF) VRDEMANDA FROM DUAL";
            $params = [
                "P_DTPROGENTAF"   => $params['DTPROGENTAF'],
                "P_CDFILDEST"     => $params['CDFILDEST'],
                "P_NRAUTOFORN"    => $params['NRDEMANDA'],
                "P_IDPERIODOORCA" => $params['IDPERIODOORCA'],
                "P_NRPERIODOORCA" => $params['NRPERIODOORCA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findDataOrcamento($params)
    {
        try {
            $query = "SELECT V_DATAINI, ADD_MONTHS(V_DATAINI,1)-1 V_DATAFIM 
                        FROM (
                            SELECT TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                         THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                         ELSE LPAD(:P_DIA,2,'0')
                                    END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY') V_DATAINI
                               FROM DATA P
                              WHERE :P_DATA BETWEEN TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                                                THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                                                ELSE LPAD(:P_DIA,2,'0')
                                                           END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY')
                                               AND ADD_MONTHS(TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                                                           THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                                                           ELSE LPAD(:P_DIA,2,'0')
                                                                      END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY'),1)-1
                              GROUP BY TO_DATE(CASE WHEN TO_NUMBER(:P_DIA) > TO_NUMBER(TO_CHAR(LAST_DAY(:P_DATA),'DD'))
                                                 THEN TO_CHAR(LAST_DAY(:P_DATA),'DD')
                                                 ELSE LPAD(:P_DIA,2,'0')
                                            END || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY'))";
            $params = [
                "P_DIA"       => $params['DIA'],
                "P_DATA"      => $params['DATA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findDiaInicioOrcamento($cdFilialDestino)
    {
        try {
            $query = "SELECT IDCONSAFORCFIL, VRPEORCBLOQFIL, NVL(P.IDVERIORCFILI,'NAO_VALIDA') IDVERIORCFILI,
                            NVL(F.NRDIAINIORCAGOP,PO.NRDIAINIORCA) DIAINIORCA, NVL(PO.CDTIPOCUSTO,'03') CDTIPOCUSTO
                        FROM FILIAL F, PARASUPRI P, PARAMORCAOPERAC PO
                        WHERE F.CDFILIAL = :P_CDFILDEST
                        AND P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";
            $params = [
                "P_CDFILDEST" => $cdFilialDestino,
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findPeriodoOrcamento($params)
    {
        try {
            $query = "SELECT MAX(NRPERIODO) V_NRPERIODOORCAGOP, MAX(IDPERIODOORCA) V_IDPERIODOORCA
                         FROM PERIODOORCAGOP P
                        WHERE TO_DATE(:V_DATAINI) BETWEEN TO_DATE(CASE WHEN TO_NUMBER(:DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(:V_DATAINI),'DD'))
                                                             THEN TO_CHAR(LAST_DAY(:V_DATAINI),'DD')
                                                             ELSE LPAD(:DIAINIORCA,2,'0')
                                                       END || TO_CHAR(P.DTINIPERIORCA,'MM') || TO_CHAR(TO_DATE(:V_DATAINI), 'YYYY'),'DD/MM/YYYY')
                                            AND ADD_MONTHS(TO_DATE(CASE WHEN TO_NUMBER(:DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(:V_DATAINI),'DD'))
                                                                        THEN TO_CHAR(LAST_DAY(:V_DATAINI),'DD')
                                                                        ELSE LPAD(:DIAINIORCA,2,'0')
                                                                   END || TO_CHAR(P.DTFIMPERIORCA,'MM') || TO_CHAR(TO_DATE(:V_DATAINI), 'YYYY'),'DD/MM/YYYY'),1)-1";
            $params = [
                "DIAINIORCA" => $params['DIAINIORCA'],
                "V_DATAINI"  => $params['V_DATAINI'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findDataEntregaAF($params)
    {
        try {
            $query = "SELECT DTPROGENTAF
                       FROM ITEMAUTF AF
                      WHERE AF.CDFILDEST  = :P_CDFILDEST
                        AND AF.CDFILLANC  = :P_CDFILLANC
                        AND AF.NRAUTOFORN = :P_NRDEMANDA
                      GROUP BY DTPROGENTAF";
            $params = [
                "P_CDFILDEST" => $params['CDFILDEST'],
                "P_CDFILLANC" => $params['CDFILLANC'],
                "P_NRDEMANDA" => $params['NRDEMANDA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findFilialAlcada($params)
    {
        try {
            $query = "SELECT
                            NRFILIALCADA,
                            NRCONFIGALCADA,
                            CDFILIAL,
                            IDCONSISTVRAPROV
                        FROM
                            FILIALCADA
                        WHERE
                            NRCONFIGALCADA = :P_NRCONFIGALCADA
                            AND CDFILIAL = :P_CDFILDEST";

            $params = [
                "P_CDFILDEST"      => $params['CDFILDEST'],
                "P_NRCONFIGALCADA" => $params['NRCONFIGALCADA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function findAprovadorAtual($params)
    {
        try {
            $query = "SELECT
                            AG.NRCONFIGALCADA,
                            AG.CDFILIAL,
                            AG.CDOPERADOR,
                            AG.VRAPROVCMP
                        FROM
                            ALCADAGERFILI   AG
                        WHERE AG.CDOPERADOR = :P_CDOPERADOR
                            AND AG.CDFILIAL = :P_CDFILDEST
                            AND EXISTS (SELECT 1
                                        FROM APROVALCADA A 
                                        WHERE A.NRDEMANDA = :P_NRDEMANDA
                                         AND A.CDOPERADOR = :P_CDOPERADOR
                                         AND A.CDFILDEST = :P_CDFILDEST
                                         AND A.NRCONFIGALCADA = AG.NRCONFIGALCADA)";

            $params = [
                "P_CDFILDEST"  => $params['CDFILDEST'],
                "P_CDOPERADOR" => $params['APROVADOR'],
                "P_NRDEMANDA"  => $params['NRAUTOFORNE'],                
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function executeCONSISTE_ORCAMENTO($unidadeDestino, $unidadeLancamento, $numeroDemanda, $dtEntrega = null, $nrorg = null)
    {
        try {
            $procedure  = new StoredProcedure($this->connection, 'CONSISTE_ORCAMENTO_GOP');
            $procedure->addParam(new Param('P_CDFILDEST',  Param::PARAM_INPUT,  $unidadeDestino,  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CDFILLANC',  Param::PARAM_INPUT,  $unidadeLancamento,  Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRAUTOFORN', Param::PARAM_INPUT,  $numeroDemanda, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_CONSISTE',   Param::PARAM_OUTPUT, NULL, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NROG',       Param::PARAM_INPUT,  $nrorg, Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DATA',       Param::PARAM_INPUT,  $dtEntrega, Param::PARAM_TYPE_STR));

            return $procedure->execute();
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaDIAINIORCA($params)
    {
        try {
            $query = "SELECT IDCONSAFORCFIL, VRPEORCBLOQFIL, 
                        NVL(P.IDVERIORCFILI,'NAO_VALIDA') IDVERIORCFILI,
                        NVL(F.NRDIAINIORCAGOP,PO.NRDIAINIORCA) DIAINIORCA, 
                        NVL(PO.CDTIPOCUSTO,'03') CDTIPOCUSTO,
                        NVL(PO.IDORCDESPDIVERS,'P') IDORCDESPDIVERS, 
                        NVL(PO.IDORCADESPMATRI,'P') IDORCADESPMATRI
                      FROM FILIAL F, PARASUPRI P, PARAMORCAOPERAC PO
                      WHERE F.CDFILIAL = :P_CDFILDEST
                       AND P.CDFILIAL = NVL(F.CDFILCENTCMP, F.CDFILIAL)";

            $params = [
                "P_CDFILDEST"     => $params['CDFILDEST'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaDTPROGENTAF($params)
    {
        try {
            $query = "SELECT DTPROGENTAF
                       FROM ITEMAUTF AF
                      WHERE AF.NRAUTOFORN = :P_NRDEMANDA
                        AND AF.CDFILDEST  = :P_CDFILDEST
                        AND AF.CDFILLANC  = :P_CDFILLANC
                      GROUP BY DTPROGENTAF";

            $params = [
                "P_NRDEMANDA" => $params['NRAUTOFORN'],
                "P_CDFILDEST" => $params['CDFILDEST'],
                "P_CDFILLANC" => $params['CDFILLANC'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaDATAINI($params)
    {
        try {
            $query = "SELECT 
                            TO_DATE( :P_DATACASE || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY') as DATAINI
                    FROM DATA P
                    WHERE :P_DATA BETWEEN 
                            TO_DATE( :P_DATACASE || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY')
                        AND ADD_MONTHS(
                            TO_DATE( :P_DATACASE || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY')
                        ,1)-1
                    GROUP BY 
                            TO_DATE( :P_DATACASE || TO_CHAR(P.DTMES,'MM/YYYY'),'DD/MM/YYYY')";

            $params = [
                "P_DATACASE" => $params['DATACASE'],
                "P_DATA"     => $params['DTPROGENTAF'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function createDATAFIM($params)
    {
        try {
            $query = "select ADD_MONTHS(:P_DATA,1)-1 as DATAFIM from filial";

            $params = [
                "P_DATA" => $params['DATAINI'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaPeriodo($params)
    {
        try {
            $query = "SELECT MAX(NRPERIODO) as MAX_NRPERIODO, 
                               MAX(IDPERIODOORCA) as MAX_IDPERIODOORCA
                        FROM PERIODOORCAGOP P
                        WHERE P.CDFILIAL = (SELECT NVL(MAX(PERIODOORCAGOP.CDFILIAL),'P') 
                        FROM PERIODOORCAGOP 
                        WHERE PERIODOORCAGOP.CDFILIAL = :P_CDFILDEST)
                            AND TO_DATE(:P_DATAINI) BETWEEN 
                                                        TO_DATE(CASE WHEN TO_NUMBER(:P_DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(TO_DATE(:P_DATAINI)),'DD'))
                                                            THEN TO_CHAR(LAST_DAY(TO_DATE(:P_DATAINI)),'DD')
                                                            ELSE LPAD(:P_DIAINIORCA,2,0)
                                                        END || TO_CHAR(P.DTINIPERIORCA,'MM') || TO_CHAR(TO_DATE(:P_DATAINI), 'YYYY'),'DD/MM/YYYY')
                                                    AND 
                                                        ADD_MONTHS(TO_DATE(CASE WHEN TO_NUMBER(:P_DIAINIORCA) > TO_NUMBER(TO_CHAR(LAST_DAY(TO_DATE(:P_DATAINI)),'DD'))
                                                            THEN TO_CHAR(LAST_DAY(TO_DATE(:P_DATAINI)),'DD')
                                                            ELSE LPAD(:P_DIAINIORCA,2,0)
                                                        END || TO_CHAR(P.DTFIMPERIORCA,'MM') || TO_CHAR(TO_DATE(:P_DATAINI), 'YYYY'),'DD/MM/YYYY'),1)-1";

            $params = [
                "P_CDFILDEST" => $params['CDFILDEST'],
                "P_DATAINI"     => $params['DATAINI'],
                "P_DIAINIORCA"     => $params['DIAINIORCA'],
            ];

            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }

    public function buscaOrcamento($params)
    {
        try {
            $query = "SELECT PROD.CDARVPROD, IDP.CDCONTCTBL, IM.DTORCAMENTO, ABS(IM.VRORCAMENTO) VRORCACONSDESP, ABS(IDP.VRORCAMENTO) VRORCAITEM,
                                          I.IDTIPOORCAOP, I.CDDESPPREV, O.IDPERIODOORCA, O.NRPERIODOORCA
                                     FROM ORCAMENTOGOP O INNER JOIN ITEMORCAGOP I ON I.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                                                 AND I.IDTIPOORCAOP IN ('CONSUMO','DESPENTRADA','DESPDIVERSA')
                                                         INNER JOIN APROVAORCAGOP A ON A.NRORCAMENTOGOP = O.NRORCAMENTOGOP
                                                                                   AND A.IDTIPOORCAOP = I.IDTIPOORCAOP
                                                         INNER JOIN ITORCAMESGOP IM ON IM.NRITEMORCAOP = I.NRITEMORCAOP
                                                         INNER JOIN ITDETALORCAMESGOP IDP ON IDP.NRITORCAMESGOP = IM.NRITORCAMESGOP
                                                         LEFT  JOIN PRODUTO PROD ON PROD.CDPRODUTO = IDP.CDPRODUTO
                                    WHERE O.CDFILIAL = :P_CDFILDEST
                                      AND O.IDPERIODOORCA = :P_IDPERIODOORCA
                                      AND O.NRPERIODOORCA = :P_NRPERIODOORCAGOP
                                      AND O.NRANOORCAGOP = TO_CHAR(TO_DATE(:P_DATAINI),'YYYY')
                                      AND :P_DTPROGENTAF BETWEEN :P_DATAINI AND :P_DATAFIM
                                      AND TO_NUMBER(TO_CHAR(TO_DATE(:P_DATAINI),'MM')) = TO_NUMBER(TO_CHAR(TO_DATE(IM.DTORCAMENTO),'MM'))
                                      AND IDP.VRORCAMENTO IS NOT NULL
                                      AND EXISTS (SELECT 1
                                                    FROM ITEMAUTF AF INNER JOIN PRODUTO P ON AF.CDPRODUTO = P.CDPRODUTO
                                                   WHERE AF.CDFILDEST  = :P_CDFILDEST
                                                     AND AF.CDFILLANC  = :P_CDFILLANC
                                                     AND AF.NRAUTOFORN = :P_NRDEMANDA
                                                     AND P.CDARVPROD LIKE PROD.CDARVPROD||'%'
                                                     AND PROD.CDARVPROD IS NOT NULL
                                                   UNION
                                                  SELECT 1
                                                    FROM ITEMAUTF AF INNER JOIN PRODUTO P ON AF.CDPRODUTO = P.CDPRODUTO
                                                   WHERE AF.CDFILDEST  = :P_CDFILDEST
                                                     AND AF.CDFILLANC  = :P_CDFILLANC
                                                     AND AF.NRAUTOFORN = :P_NRDEMANDA
                                                     AND CTBL_PRODUTO(AF.CDPRODUTO,'E') LIKE IDP.CDCONTCTBL||'%'
                                                     AND IDP.CDCONTCTBL IS NOT NULL
                                                     AND (NOT EXISTS (SELECT PPR.CDFILIAL
                                                                        FROM PRODUTOROP PPR, PRODUTO PP
                                                                       WHERE PPR.IDTPPRODUTOROP = 'CO'
                                                                         AND PPR.CDFILIAL = TIPOPARAM_DRO(:P_CDFILDEST)
                                                                         AND PPR.CDPRODUTO = PP.CDPRODUTO
                                                                         AND P.CDARVPROD LIKE PP.CDARVPROD||'%')
                                                          OR EXISTS (SELECT PPR.CDFILIAL
                                                                       FROM PRODUTOROP PPR, PRODUTO PP
                                                                      WHERE PPR.IDTPPRODUTOROP = 'OX'
                                                                        AND PPR.CDFILIAL = TIPOPARAM_DRO(:P_CDFILDEST)
                                                                        AND PPR.CDPRODUTO = PP.CDPRODUTO
                                                                        AND P.CDARVPROD LIKE PP.CDARVPROD||'%')))
                                  ORDER BY I.IDTIPOORCAOP, I.CDDESPPREV, PROD.CDARVPROD, IDP.CDCONTCTBL DESC";

            $params = [
                "P_CDFILDEST"        => $params['CDFILDEST'],
                "P_CDFILLANC"        => $params['CDFILLANC'],
                "P_NRDEMANDA"        => $params['NRAUTOFORN'],
                "P_DATAINI"          => $params['DATAINI'],
                "P_DATAFIM"          => $params['DATAFIM'],
                "P_DTPROGENTAF"      => $params['DTPROGENTAF'],
                "P_IDPERIODOORCA"    => $params['IDPERIODOORCA'],
                "P_NRPERIODOORCAGOP" => $params['NRPERIODO'],
            ];
            return $this->supUtilities->crud($query, $params);
        } catch (\Exception $e) {
            throw $e;
        }
    }
}
